package cn.com.qmth.examcloud.core.oe.task.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcExamScoreReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcFaceBiopsyResultReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcExamScoreResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcFaceBiopsyResultResp;
import cn.com.qmth.examcloud.core.oe.task.dao.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.task.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.task.service.bean.CalculateFaceCheckResultInfo;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.enums.IsSuccess;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Description 考试记录服务
 * @Author lideyin
 * @Date 2019/12/18 14:37
 * @Version 1.0
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private RedisClient redisClient;
    @Autowired
    private ExamCaptureQueueRepo examCaptureQueueRepo;
    @Autowired
    private ExamCaptureService examCaptureService;
    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    @Override
    public void saveExamRecordDataCache(Long examRecordDataId, ExamRecordData data) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.set(key, data, 2592000);
    }

    @Override
    public ExamRecordData getExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        return redisClient.get(key, ExamRecordData.class);
    }

    @Override
    public void deleteExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.delete(key);
    }

    /**
     * 计算自动审核结果
     *
     * @param isNoPhotoAndIllegality
     * @param faceVerifyResult
     * @param isIllegality
     * @return
     */
    @Override
    public Boolean calcAutoAuditResult(Boolean isNoPhotoAndIllegality, IsSuccess faceVerifyResult, Boolean isIllegality) {
        //无照片违纪自动审核
        if (isNoPhotoAndIllegality != null && isNoPhotoAndIllegality) {
            return true;
        }

        //活体检测失败违纪自动审核
        if (null != faceVerifyResult && IsSuccess.FAILED == faceVerifyResult && null != isIllegality && isIllegality) {
            return true;
        }

        return null;
    }

    /**
     * 交卷后续处理
     *
     * @param examRecordDataId
     * @return
     */
    @Override
    public ExamRecordData processAfterHandInExam(Long examRecordDataId) {
        ExamRecordData examRecordData = getExamRecordDataCache(examRecordDataId);

        //只有开启的人脸的才会进行如下处理
        if (FaceBiopsyHelper.isFaceEnable(examRecordData.getRootOrgId(),
                examRecordData.getExamId(), examRecordData.getStudentId())) {
            // 判断是否存在未处理的图片
            boolean existUnhandledExamCaptureQueue = (examCaptureQueueRepo
                    .existsUnhandledByExamRecordDataId(examRecordDataId) != null);
            if (existUnhandledExamCaptureQueue) {
                throw new StatusException(Constants.CAPTURE_PROCESSING_STATUS_CODE, "PROCESSING");
            }

            // 计算人脸检测结果
            CalculateFaceCheckResultInfo faceCheckResult = examCaptureService.calculateFaceCheckResult(examRecordDataId);
            modifyExamRecordDataByFaceCheckResult(examRecordData, faceCheckResult);

            // 计算活体检测结果
            CalcFaceBiopsyResultReq req = new CalcFaceBiopsyResultReq();
            req.setExamRecordDataId(examRecordDataId);
            CalcFaceBiopsyResultResp calcFaceBiopsyResultResp = examRecordDataCloudService.calcFaceBiopsyResult(req);
            modifyExamRecordDataByFaceBiopsyResult(examRecordData, calcFaceBiopsyResultResp);

            // 违纪自动审核
            Boolean isAudit = this.calcAutoAuditResult(faceCheckResult.getNoPhotoAndIllegality(),
                    calcFaceBiopsyResultResp.getFaceVerifyResult(), calcFaceBiopsyResultResp.getIllegality());
            if (null != isAudit) {
                examRecordData.setIsAudit(isAudit);
            }
        }

        //自动计算客观分
        CalcExamScoreReq cesReq = new CalcExamScoreReq();
        cesReq.setExamRecordDataId(examRecordDataId);
        CalcExamScoreResp calcExamScoreResp = examRecordDataCloudService.calcExamScore(cesReq);
        examRecordData.setObjectiveScore(calcExamScoreResp.getObjectiveScore());
        examRecordData.setObjectiveAccuracy(calcExamScoreResp.getObjectiveAccuracy());
        examRecordData.setSuccPercent(calcExamScoreResp.getSuccPercent());
        examRecordData.setTotalScore(calcExamScoreResp.getTotalScore());

        // 更新考试状态
        examRecordData.setExamRecordStatus(
                examRecordData.getHandInExamType() == HandInExamType.MANUAL
                        ? ExamRecordStatus.EXAM_END
                        : ExamRecordStatus.EXAM_OVERDUE);

        //更新考试记录中的相关数据
        this.saveExamRecordDataCache(examRecordDataId, examRecordData);

        return examRecordData;
    }

    private void modifyExamRecordDataByFaceBiopsyResult(ExamRecordData examRecordData, CalcFaceBiopsyResultResp calcFaceBiopsyResultResp) {
        if (null != calcFaceBiopsyResultResp.getIllegality()) {
            examRecordData.setIsIllegality(calcFaceBiopsyResultResp.getIllegality());
        }
        if (null != calcFaceBiopsyResultResp.getWarn()) {
            examRecordData.setIsWarn(calcFaceBiopsyResultResp.getWarn());
        }
        if (null != calcFaceBiopsyResultResp.getFaceVerifyResult()) {
            examRecordData.setFaceVerifyResult(calcFaceBiopsyResultResp.getFaceVerifyResult());
        }
    }

    private void modifyExamRecordDataByFaceCheckResult(ExamRecordData examRecordData, CalculateFaceCheckResultInfo faceCheckResult) {
        if (null != faceCheckResult.getWarn()) {
            examRecordData.setIsWarn(faceCheckResult.getWarn());
        }
        if (null != faceCheckResult.getIllegality()) {
            examRecordData.setIsIllegality(faceCheckResult.getIllegality());
        }
        if (null != faceCheckResult.getFaceTotalCount()) {
            examRecordData.setFaceTotalCount(faceCheckResult.getFaceTotalCount());
        }
        if (null != faceCheckResult.getFaceSuccessPercent()) {
            examRecordData.setFaceSuccessPercent(faceCheckResult.getFaceSuccessPercent());
        }
        if (null != faceCheckResult.getFaceStrangerCount()) {
            examRecordData.setFaceStrangerCount(faceCheckResult.getFaceStrangerCount());
        }
        if (null != faceCheckResult.getFaceSuccessCount()) {
            examRecordData.setFaceSuccessCount(faceCheckResult.getFaceSuccessCount());
        }
        if (null != faceCheckResult.getFaceFailedCount()) {
            examRecordData.setFaceFailedCount(faceCheckResult.getFaceFailedCount());
        }
        if (null != faceCheckResult.getBaiduFaceLivenessSuccessPercent()) {
            examRecordData.setBaiduFaceLivenessSuccessPercent(faceCheckResult.getBaiduFaceLivenessSuccessPercent());
        }
    }
}
